#include "fdbuf.h"
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

///////////////////////////////////////////////////////////////////////////////
// Class fdobuf
///////////////////////////////////////////////////////////////////////////////
fdobuf::fdobuf(int fdesc, bool dc, unsigned bufsize)
  : fdbuf(fdesc, dc, bufsize)
{
}

fdobuf::fdobuf(const char* filename, fdobuf::openflags f, unsigned bufsize)
  : fdbuf(open(filename, O_WRONLY | f, 0666), true, bufsize)
{
  if(fd == -1) {
    flags = flag_error;
    errnum = errno;
  }
}

fdobuf::~fdobuf()
{
  flush();
}

bool fdobuf::operator!() const
{
  return error() || closed();
}

bool fdobuf::flush()
{
  if(flags)
    return false;
  while(start < length) {
    ssize_t written = ::write(fd, buf+start, length-start);
    if(written == -1) {
      flags |= flag_error;
      errnum = errno;
      return false;
    }
    else
      start += written;
  }
  length = 0;
  start = 0;
  return true;
}

bool fdobuf::write(const char* data, unsigned datalen)
{
  const char* ptr = data;
  count = 0;
  while(datalen) {
    unsigned amount = size-length;
    if(amount > datalen)
      amount = datalen;
    memcpy(buf+length, ptr, amount);
    length += amount;
    datalen -= amount;
    ptr += amount;
    if(length >= size && !flush())
      return false;
    count += amount;
  }
  return true;
}

bool fdobuf::writeln()
{
  return write("\n", 1) &&
    flush();
}

bool fdobuf::writeln(const char* str)
{
  return write(str, strlen(str)) &&
    write("\n", 1) &&
    flush();
}

bool fdobuf::writeln(const mystring& str)
{
  return write(str.c_str(), str.length()) &&
    write("\n", 1) &&
    flush();
}

///////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////////
fdobuf fout(1);
fdobuf ferr(2);
